@@ -1,6 +1,6 @@
 # Orca
 #
-# Copyright 2004-2005 Sun Microsystems Inc.
+# Copyright 2004-2006 Sun Microsystems Inc.
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Library General Public
@@ -19,12 +19,11 @@
 
 import gtk
 import os, signal, sys
+import time
 
 import atspi
 import braille
 import debug
-import focus_tracking_presenter
-import hierarchical_presenter
 import keynames
 import keybindings
 #import mag - [[[TODO: WDW - disable until I can figure out how to
@@ -32,6 +31,7 @@ import keybindings
 #             as bugzilla bug 319643.]]]
 import settings
 import speech
+import threading
 
 from input_event import BrailleEvent
 from input_event import KeyboardEvent
@@ -59,10 +59,9 @@ MODIFIER_ORCA = 8
 #                                                                      #
 ########################################################################
 
-# The known presentation managers.
+# The known presentation managers (set up in start())
 #
-_PRESENTATION_MANAGERS = [focus_tracking_presenter.FocusTrackingPresenter(),
-                          hierarchical_presenter.HierarchicalPresenter()]
+_PRESENTATION_MANAGERS = None
 
 # The current presentation manager, which is an index into the
 # _PRESENTATION_MANAGERS list.
@@ -216,7 +215,7 @@ def findActiveWindow():
             state = app.child(i).state
             if state.count(atspi.Accessibility.STATE_ACTIVE) > 0:
                 window = app.child(i)
-		break
+                break
 
     return window
 
@@ -233,13 +232,13 @@ def _buildAppList(registry):
     apps = []
 
     for i in range(0, registry.desktop.childCount):
-        acc = registry.desktop.getChildAtIndex(i)
         try:
+            acc = registry.desktop.getChildAtIndex(i)
             app = atspi.Accessible.makeAccessible(acc)
             if app:
                 apps.insert(0, app)
         except:
-            debug.printException(debug.LEVEL_SEVERE)
+            debug.printException(debug.LEVEL_FINEST)
 
     debug.println(debug.LEVEL_FINEST,
                   "...orca._buildAppList")
@@ -271,7 +270,13 @@ def _onChildrenChanged(e):
         # apps[] list.  If this logic is changed in the future, the apps list
         # will most likely needed to be updated here.]]]
         #
-        _buildAppList(registry)
+        # [[[WDW - 03/07/2006 - comment this out until we can figure out
+        # why Orca sometimes hangs here.  The main hang seems to be when
+        # Orca calls getChildAtIndex on the registry.  The hierarchical
+        # presenter depends upon this list, though, so I've commented it
+        # out for now as well.]]]
+        #
+        #_buildAppList(registry)
 
 ########################################################################
 #                                                                      #
@@ -375,6 +380,62 @@ def toggleKeystrokeRecording(script=None, inputEvent=None):
 #                                                                      #
 ########################################################################
 
+def cycleDebugLevel(script=None, inputEvent=None):
+    """Cycles the debug level at run time.
+
+    Arguments:
+    - inputEvent: the InputEvent instance that caused this to be called.
+
+    Returns True indicating the event should be consumed.
+    """
+
+def cycleDebugLevel(script=None, inputEvent=None):
+    global _debugLevel
+
+    level = debug.getDebugLevel()
+    
+    if level == debug.LEVEL_ALL:
+        level = debug.LEVEL_FINEST
+    elif level == debug.LEVEL_FINEST:
+        level = debug.LEVEL_FINER
+    elif level == debug.LEVEL_FINER:
+        level = debug.LEVEL_FINE
+    elif level == debug.LEVEL_FINE:
+        level = debug.LEVEL_CONFIGURATION
+    elif level == debug.LEVEL_CONFIGURATION:
+        level = debug.LEVEL_INFO
+    elif level == debug.LEVEL_INFO:
+        level = debug.LEVEL_WARNING
+    elif level == debug.LEVEL_WARNING:
+        level = debug.LEVEL_SEVERE
+    elif level == debug.LEVEL_SEVERE:
+        level = debug.LEVEL_OFF
+    elif level == debug.LEVEL_OFF:
+        level = debug.LEVEL_ALL
+
+    debug.setDebugLevel(level)
+    
+    if level == debug.LEVEL_ALL:
+        speech.speak(_("Debug level all."))
+    elif level == debug.LEVEL_FINEST:
+        speech.speak(_("Debug level finest."))
+    elif level == debug.LEVEL_FINER:
+        speech.speak(_("Debug level finer."))
+    elif level == debug.LEVEL_FINE:
+        speech.speak(("Debug level fine."))
+    elif level == debug.LEVEL_CONFIGURATION:
+        speech.speak("Debug level configuration.")
+    elif level == debug.LEVEL_INFO:
+        speech.speak("Debug level info.")
+    elif level == debug.LEVEL_WARNING:
+        speech.speak("Debug level warning.")
+    elif level == debug.LEVEL_SEVERE:
+        speech.speak("Debug level severe.")
+    elif level == debug.LEVEL_OFF:
+        speech.speak("Debug level off.")
+
+    return True
+
 def printApps(script=None, inputEvent=None):
     """Prints a list of all applications to stdout
 
@@ -398,51 +459,6 @@ def printApps(script=None, inputEvent=None):
 
     return True
 
-def printAccessibleTree(level, indent, root):
-    debug.printDetails(level, indent, root, False)
-    for i in range(0, root.childCount):
-        child = root.child(i)
-        if child == root:
-            debug.println(level,
-                          indent + "  " + "WARNING CHILD == PARENT!!!")
-        elif not child:
-            debug.println(level,
-                          indent + "  " + "WARNING CHILD IS NONE!!!")
-        elif child.parent != root:
-            debug.println(level,
-                          indent + "  " + "WARNING CHILD.PARENT != PARENT!!!")
-        else:
-            printAccessibleTree(level, indent + "  ", child)
-
-def printAccessiblePaintedTree(level, indent, root):
-    debug.printDetails(level, indent, root, False)
-
-    extents = root.extents
-    if extents:
-        debug.println(level, \
-                      indent + " extents: (x=%d y=%d w=%d h=%d)" \
-                      % (extents.x, extents.y, extents.width, extents.height))
-
-    #if root.text:
-    #    extents = root.text.getCharacterExtents(0,0)
-    #    debug.println(level, \
-    #                  indent + " text extents: (x=%d y=%d w=%d h=%d)" \
-    #                  % (extents[0], extents[1], extents[2], extents[3]))
-
-    for i in range(0, root.childCount):
-        child = root.child(i)
-        if child == root:
-            debug.println(level,
-                          indent + "  " + "WARNING CHILD == PARENT!!!")
-        elif not child:
-            debug.println(level,
-                          indent + "  " + "WARNING CHILD IS NONE!!!")
-        elif child.parent != root:
-            debug.println(level,
-                          indent + "  " + "WARNING CHILD.PARENT != PARENT!!!")
-        elif child.state.count(atspi.Accessibility.STATE_SHOWING):
-            printAccessiblePaintedTree(level, indent + "  ", child)
-
 def printActiveApp(script=None, inputEvent=None):
     """Prints the active application.
 
@@ -467,6 +483,17 @@ def printActiveApp(script=None, inputEvent=None):
 
     return True
 
+def printAncestry(script=None, inputEvent=None):
+    """Prints the ancestry for the current locusOfFocus"""
+    atspi.printAncestry(locusOfFocus)
+    return True
+
+def printHierarchy(script=None, inputEvent=None):
+    """Prints the application for the current locusOfFocus"""
+    if locusOfFocus:
+        atspi.printHierarchy(locusOfFocus.app, locusOfFocus)
+    return True
+
 ########################################################################
 #                                                                      #
 # METHODS FOR HANDLING LEARN MODE.                                     #
@@ -594,12 +621,12 @@ def _processKeyboardEvent(event):
         # We treat the Insert key as a modifier - so just swallow it and
         # set our internal state.
         #
-        if event_string == "Insert":
+        if (event_string == "Insert") or (event_string == "KP_Insert"):
             _insertPressed = True
             return True
 
     elif event.type == atspi.Accessibility.KEY_RELEASED_EVENT \
-         and (event_string == "Insert"):
+         and ((event_string == "Insert") or (event_string == "KP_Insert")):
         _insertPressed = False
         return True
 
@@ -823,13 +850,31 @@ def init(registry):
                                             0,
                                             keystrokeRecordingHandler))
 
+    resetSpeechHandler = InputEventHandler(\
+        resetSpeech,
+        _("Forces a reset of the speech service."))
+    _keybindings.add(keybindings.KeyBinding("s", \
+                                            1 << MODIFIER_ORCA, \
+                                            1 << MODIFIER_ORCA, \
+                                            resetSpeechHandler))
+
     listAppsHandler = InputEventHandler(
         printApps,
         _("Prints a debug listing of all known applications to the console where Orca is running."))
-    _keybindings.add(keybindings.KeyBinding("F5", \
-                                            1 << MODIFIER_ORCA, \
-                                            1 << MODIFIER_ORCA,
-                                            listAppsHandler))
+    _keybindings.add(keybindings.KeyBinding(
+        "F5", 
+        (1 << MODIFIER_ORCA | 1 << atspi.Accessibility.MODIFIER_CONTROL), 
+        1 << MODIFIER_ORCA,
+        listAppsHandler))
+
+    cycleDebugLevelHandler = InputEventHandler(
+        cycleDebugLevel,
+        _("Cycles the debug level at run time."))
+    _keybindings.add(keybindings.KeyBinding(
+        "F5", 
+        (1 << MODIFIER_ORCA | 1 << atspi.Accessibility.MODIFIER_CONTROL), 
+        (1 << MODIFIER_ORCA | 1 << atspi.Accessibility.MODIFIER_CONTROL), 
+        cycleDebugLevelHandler))
 
     printActiveAppHandler = InputEventHandler(\
         printActiveApp,
@@ -839,6 +884,25 @@ def init(registry):
                                             1 << MODIFIER_ORCA,
                                             printActiveAppHandler))
 
+    printAncestryHandler = InputEventHandler(\
+        printAncestry,
+        _("Prints debug information about the ancestry of the object with focus"))
+    _keybindings.add(keybindings.KeyBinding(
+        "F7",
+        (1 << MODIFIER_ORCA | 1 << atspi.Accessibility.MODIFIER_CONTROL), 
+        1 << MODIFIER_ORCA,
+        printAncestryHandler))
+
+    printHierarchyHandler = InputEventHandler(\
+        printHierarchy,
+        _("Prints debug information about the application with focus"))
+    
+    _keybindings.add(keybindings.KeyBinding(
+        "F7",
+        (1 << MODIFIER_ORCA | 1 << atspi.Accessibility.MODIFIER_CONTROL), 
+        (1 << MODIFIER_ORCA | 1 << atspi.Accessibility.MODIFIER_CONTROL), 
+        printHierarchyHandler))
+
     nextPresentationManagerHandler = InputEventHandler(\
         _switchToNextPresentationManager,
         _("Switches to the next presentation manager."))
@@ -859,7 +923,7 @@ def init(registry):
         debug.println(debug.LEVEL_CONFIGURATION,
                       "Speech module has NOT been initialized.")
 
-    if settings.getSetting(settings.USE_BRAILLE, False):
+    if settings.getSetting(settings.USE_BRAILLE, True):
         try:
             braille.init(_processBrailleEvent, 7)
         except:
@@ -885,15 +949,25 @@ def init(registry):
     # Create and load an app's script when it is added to the desktop
     #
     registry.registerEventListener(_onChildrenChanged,
-				   "object:children-changed:")
+                                   "object:children-changed:")
 
     _initialized = True
     return True
 
+def resetSpeech(script=None, inputEvent=None):
+    """Kicks the speech server in the pants and re-establishes a
+    connection to it."""
+    speech.reset()
+    time.sleep(1) # give the driver a chance to die.
+    speech.reset("Speech has been reset.")    
+    return True
+
 def start(registry):
     """Starts Orca.
     """
 
+    global _PRESENTATION_MANAGERS
+    
     if not _initialized:
         init(registry)
 
@@ -903,10 +977,32 @@ def start(registry):
     except:
         debug.printException(debug.LEVEL_SEVERE)
 
+    if not _PRESENTATION_MANAGERS:
+        # [[[WDW - comment out hierarchical_presenter for now.  It relies
+        # on orca.apps, and we've disabled that due to a hang in a call
+        # to getChildAtIndex in _buildAppList.]]]
+        #
+        #import focus_tracking_presenter
+        #import hierarchical_presenter
+        #_PRESENTATION_MANAGERS = \
+        #    [focus_tracking_presenter.FocusTrackingPresenter(),
+        #     hierarchical_presenter.HierarchicalPresenter()]
+
+        import focus_tracking_presenter
+        _PRESENTATION_MANAGERS = \
+            [focus_tracking_presenter.FocusTrackingPresenter()]
+    
     _switchToPresentationManager(0) # focus_tracking_presenter
 
     registry.start()
 
+def abort():
+    sys.exit(1)
+
+def timeout():
+    print "TIMEOUT: Hung while trying to shutdown.  Aborting."
+    abort()
+    
 def shutdown(script=None, inputEvent=None):
     """Exits Orca.  Unregisters any event listeners and cleans up.  Also
     quits the bonobo main loop and resets the initialized state to False.
@@ -920,6 +1016,13 @@ def shutdown(script=None, inputEvent=None):
     if not _initialized:
         return False
 
+    # [[[TODO: WDW - the timer stuff is an experiment to see if
+    # we can recover from hangs.  It's only experimental, so it's
+    # commented out for now.]]]
+    #
+    #timer = threading.Timer(5.0, timeout)
+    #timer.start()
+    
     speech.speak(_("goodbye."))
     braille.displayMessage(_("Goodbye."))
 
@@ -927,7 +1030,7 @@ def shutdown(script=None, inputEvent=None):
     #
     registry = atspi.Registry()
     registry.deregisterEventListener(_onChildrenChanged,
-            	                     "object:children-changed:")
+                                     "object:children-changed:")
 
     if _currentPresentationManager >= 0:
         _PRESENTATION_MANAGERS[_currentPresentationManager].deactivate()
@@ -943,25 +1046,61 @@ def shutdown(script=None, inputEvent=None):
 
     registry.stop()
 
+    #timer.cancel()
+    #del timer
+    
     _initialized = False
     return True
 
-def shutdownAndExit(signum, frame):
-    print "Goodbye."
+exitCount = 0
+def shutdownOnSignal(signum, frame):
+    global exitCount
+    
+    print "Shutting down and exiting due to signal =", signum
+    
+    # Well...we'll try to exit nicely, but if we keep getting called,
+    # something bad is happening, so just quit.
+    #
+    if exitCount:
+        abort()
+    else:
+        exitCount += 1
+
+    # Try to do a graceful shutdown if we can.
+    #
     try:
-	shutdown()
+        shutdown()
+        return
     except:
-        pass
-    sys.exit()
+        abort()
+
+def abortOnSignal(signum, frame):
+    print "Aborting due to signal =", signum
+    abort()
 
 def main():
-    userprefs = os.path.join (os.environ["HOME"], ".orca")
-    sys.path.insert (0, userprefs)
-    signal.signal(signal.SIGINT, shutdownAndExit)
-    signal.signal(signal.SIGQUIT, shutdownAndExit)
+    #import commands
+    #a11yEnabled = commands.getoutput(\
+    #    "gconftool-2 --get /desktop/gnome/interface/accessibility")
+    #if a11yEnabled != "true":
+    #    print _("Accessibility has not been enabled for this session.")
+    #    print _("Please run orca-setup and then logout and log back in.")
+    #    abort()
+        
+    userprefs = os.path.join(os.environ["HOME"], ".orca")
+    sys.path.insert(0, userprefs)
+    sys.path.insert(0, '') # current directory
+
+    signal.signal(signal.SIGHUP, shutdownOnSignal)
+    signal.signal(signal.SIGINT, shutdownOnSignal)
+    signal.signal(signal.SIGTERM, shutdownOnSignal)
+    signal.signal(signal.SIGQUIT, shutdownOnSignal)
+    signal.signal(signal.SIGSEGV, abortOnSignal)
+    
     registry = atspi.Registry()
     init(registry)
     start(registry)
+    return 0
 
 if __name__ == "__main__":
-    main()
+    sys.exit(main())
